<?php
namespace Plugin\DBMS;

use Plugin\Exception\Error;
class Manager {
	
	private $con;
	
	/**
	 * Array of datamanagers currently in memory
	 * @var unknown
	 */
	private $dataManagers;
	
	/**
	 * Connection object
	 * @return \mysqli
	 */
	public function getConnection(){
		return MySQLHelper::getConnection();
	}
	
	public static function getManager($con = null){
		self::connect($con);
		return new self;
	}
	
	public static function connect($con = null){
		MySQLHelper::connect($con);
	}
	
	public function close(){
		MySQLHelper::close();
	}
	
	/**
	 * Get the data from a datamanager
	 * use cache and expiration will be used later when we implement memcache
	 * @param string $dataManagerName Ex: Blog:Model:Article
	 * @param string $useCache
	 * @param number $expiration
	 * 
	 * @return \Plugin\DBMS\Interaction;
	 */
	public function get($dataManagerName,$useCache = false,$expiration = 0){
		if (isset($this->dataManagers[$dataManagerName])){
			return $this->dataManagers[$dataManagerName];
		}
		$path = explode(":", $dataManagerName);
		$class = "src\\".$path[0]."\\".$path[1]."\\Interaction\\".$path[2]."DataManager";
		
		if (!class_exists($class)){
			Error::throw500("The datamanager class $class does not exist");
		}
		$dbm = new $class;
		$this->dataManagers[$dataManagerName] = $dbm;
		return $dbm;
		
	}
	
	public function rawQuery($sql){
		$con = $this->getConnection();
		$result = MySQLHelper::query($con, $sql);
		
		if (!$result || (isset($result->field_count) && $result->field_count === 0)){
			return false;
		}
		
		$group = array();
		
		//TODO: This is not safe, need to check what is the value when it fails
		while(($r = $result->fetch_object()) != false) {
			$group[] = $r;
		}
		return $group;
	}
}