<?php
namespace Plugin\Routing;

use Plugin\Exception\Error;
use App\Config\Settings;

class Router {
	
	private static $rt;
	
	private static $routes;
	private static $foundRoutes = array();
	
	private $uri;
	private $currentRoute;
	
	private static function initialize(){
		if (self::$rt){
			return ;
		}
		$rt = new self();
		
		$app = Settings::getAppsInfo();
		$routePaths = $rt->getAppPaths($app);
		
		$allRoutes = array();
		
		foreach ($routePaths as $path){
			$filepath = ROOT_DIR."{$path}";
			if (file_exists($filepath)){
				include($filepath); // $routes variable is in the include
				$allRoutes = array_merge($allRoutes,$routes);
			}
		}
		
		if (empty($allRoutes)){
			Error::throw500("There are no routes availabe. Go to src/{Your App}/{Section}/Config/routes.php");
		}
		
		$rt->setUri($_SERVER["REQUEST_URI"]);
		$rt->setRoutes($allRoutes);
		
		self::$rt = $rt;
	}
	
	public static function getRoute(){
		self::initialize();
		$rt = self::$rt;
		$rt->processRoute();
		return $rt->getCurrentRoute();
	}
	
	public function setUri($uri){
		$this->uri = $uri;
	}
	
	public function setRoutes($routes){
		self::$routes = $routes;
	}
	
	public function getCurrentRoute(){
		return $this->currentRoute;
	}
	
	public function processRoute(){
		$path = explode("/",$this->uri);
		$path_length = count($path);
		
		$match = false;
		$theMatch = null;
		$vars = array();
		$mat = array();
		foreach (self::$routes as $l) {
			
			foreach($l as $name => $config){
				
				if ($match){
					break;
				}
				$invars = array();
				//echo "$name\n";
				$pattern = $l['_pattern'];
				$pattern_path = explode("/",$pattern);
		
				$c = 1;
				$patterna = preg_replace_callback("/(\{.*\})/U", function ($match) use (&$invars){
					$invars[] = str_replace("{", "", str_replace("}", "", $match[1]));
					return "(.*)";
				}, $pattern);
				//print_r($vars);
				if (preg_match("~^$patterna$~", $this->uri,$mematch)){
					array_shift($mematch);
					$values = array();
					$cTotal = count($mematch);
					for($i=0;$i<$cTotal;$i++){
						$values[$invars[$i]] = $mematch[$i];
					}
					$theMatch = array ('name'=> $name,"config"=>$config,"values"=>$values);
					$match = true;
					$this->currentRoute = $theMatch;
					$vars[] = $invars;
					unset($invars);
					break;
				}
		
			}
			if ($match){
				break;
			}
		}
		
		if ($theMatch){
			// It worked
		}else {
			Error::throw404("The current page cannot be found, Url does not match any controller");
		}
	}
	
	public static function getTemplatePath($path){
		list($app,$name,$template) = explode(":",$path);
		$filepath = "src/{$app}/{$name}/template/layout/$template";
		return $filepath;
	}
	
	/**
	 * Create a url from its rout name
	 *
	 * Set the third parameter to true to return the full Url
	 * @param string $name
	 * @param array $params
	 * @param boolean $fullUrl Return the full url or just relative
	 * @return boolean|string
	 */
	public static function getUrlFromRouteName($name,$params= array(),$fullUrl = false){
		$current = array();
		$currentName = "";
		$host = "";
		if ($fullUrl){
			$protocol = ((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
			$host = $protocol.(isset($_SERVER['HTTP_HOST'])?$_SERVER['HTTP_HOST']:"nohost");
		}
	
		$stop = false;
		if (isset(self::$foundRoutes[$name])){ // recycle if it is already found
			$current = self::$foundRoutes[$name];
		}else {
			
			foreach (self::$routes as $key => $group) {
				if ($name === $key){
					$current = self::$routes[$name];
					self::$foundRoutes[$name] = $current;
					break;
				}
				
				foreach ($group as $n => $config){
					//echo "Name: $n\n";
					if ($name == $n){
						$current = $group[$n];
						$stop = true;
						break;
					}
					self::$foundRoutes[$n] = $group[$n]; // recycle for faster access next time
				}
				if ($stop){
					break;
				}
			}
		}
		if (empty($current)){
			Error::throw404("Cannot generate url from Route: $name\n");
		}
		$url = $current['_pattern'];
		$used = array();
		$extra = array();
		$readyUrl = preg_replace_callback("/(\{.*\})/U", function($a) use ($params,&$used) {
			// TODO: Complete the routing for params
			$pname = str_replace("{", "", str_replace("}", "", $a[1]));
				
			if (isset($params[$pname])){
				$used[]= $pname;
				return rawurlencode($params[$pname]);
			}
				
		}, $url);
		//print_r($used);
		foreach ($params as $key => $value){
			if (!in_array($key, $used)){
				$extra[] = $key."=".urlencode($value);
			}
		}
		$extraStr = !empty($extra)?"?".implode("&",$extra):"";
		return "{$host}{$readyUrl}{$extraStr}";
	}
	
	
	private function getAppPaths($apps){
		$path = array();
		foreach ($apps as $app => $sections) {
			foreach($sections as $name){
				$path[] = "/src/$app/$name/Config/routes.php";
			}
		}
		return $path;
	}
}