<?php
namespace Plugin\Server;

use Plugin\Routing\Router;
use Plugin\View\View;
class Response {
	
	/**
	 * Headers to be sent with the respons:
	 * In key value format
	 * 
	 * Example: array("Content-Type" => "application/json");
	 * @var array $headers
	 */
	private $headers = array();
	
	/**
	 * string content to be sent as a response
	 * @var string
	 */
	private $content;
	
	/**
	 * Absolute path of the template to run
	 * @var string
	 */
	private $template;
	
	/**
	 * Array of variables to be accessible inside the template;
	 * @var array
	 */
	private $vars;
	
	
	public function __construct($content = null){
		$this->content = $content;
	}
	
	public function sendHeaders(){
		if (empty($this->headers)){
			// Nothing to see here
			return ;
		}
		
		foreach($this->headers as $key => $value){
			if (!empty($value)){
				header("$key: $value");
			}else {
				header("$key");
			}
		}
	}
	
	public function setContent($content){
		$this->content = $content;
	}
	
	/**
	 * Set the template in colon separated format:
	 * Blog:Main:index.html.php
	 * => src/Blog/Main/template/index.html.php
	 * 
	 * Variables will be extracted and available  using the extract() function
	 * @param string $path
	 * @param array $vars
	 */
	public function setTemplate($path,$vars = array()){
		$this->template = Router::getTemplatePath($path);
		$this->vars = $vars;
	}
	
	/**
	 * If the content is already set, just print it and exit
	 */
	public function send(){
		///echo $this->content;
		if (!empty($this->content)){
			echo $this->content;
			return;
		}
		$content = "";
		if (!empty($this->template)){
			$view = View::getView($this->template,$this->vars);
			echo $view->getContent();
		}
			
	}
}